<?php
/**
 * Created by PhpStorm.
 * User: 20420
 * Date: 2020/12/8
 * Time: 16:35
 */

namespace JiuYangPErp\Bizs;


use function foo\func;

class CharactersChange
{
    /**
     * unicode编码转换成字符
     * @param int $unicode
     * @return string UTF-8字符
     **/
    private static function unicode2Char($unicode)
    {
        if($unicode < 128)     return chr($unicode);
        if($unicode < 2048)    return chr(($unicode >> 6) + 192) .
            chr(($unicode & 63) + 128);
        if($unicode < 65536)   return chr(($unicode >> 12) + 224) .
            chr((($unicode >> 6) & 63) + 128) .
            chr(($unicode & 63) + 128);
        if($unicode < 2097152) return chr(($unicode >> 18) + 240) .
            chr((($unicode >> 12) & 63) + 128) .
            chr((($unicode >> 6) & 63) + 128) .
            chr(($unicode & 63) + 128);
        return false;
    }

    /**
     * 字符转换成unicode编码
     * @param string $char 必须是UTF-8字符
     * @return int
     **/
    private static function char2Unicode($char)
    {
        switch (strlen($char)){
            case 1 : return ord($char);
            case 2 : return (ord($char{1}) & 63) |
                ((ord($char{0}) & 31) << 6);
            case 3 : return (ord($char{2}) & 63) |
                ((ord($char{1}) & 63) << 6) |
                ((ord($char{0}) & 15) << 12);
            case 4 : return (ord($char{3}) & 63) |
                ((ord($char{2}) & 63) << 6) |
                ((ord($char{1}) & 63) << 12) |
                ((ord($char{0}) & 7)  << 18);
            default :
                trigger_error('Character is not UTF-8!', E_USER_WARNING);
                return false;
        }
    }

    /**
     * 半角转全角
     * @param string $str
     * @return string
     **/
    public static function half2full($str, $pattern=''){
        //如果空的替换字符则对字符串全部替换
        if(empty($pattern)) {
            $pattern  = '/([\x{0020}\x{0020}-\x{2F}\x{3A}-\x{0040}\x{5B}-\x{0060}\x{7B}-\x{7E}]){1}/ui';
        }
        
        return  preg_replace_callback(
        // 半角字符
            $pattern,
            // 编码转换
            // 0x0020是空格，特殊处理，其他半角字符编码+0xfee0即可以转为全角
            function ($match){
                $code = self::char2Unicode($match[0]);
                if( $code == 0x0020) {
                    return self::unicode2Char(0x3000);
                }
                $code+=0xfee0;
                return self::unicode2Char($code);
            },
            $str
        );
    }

    /**
     * @param $str
     * @param $replace
     * @return string
     * 对系统默认的非法字符转义并返回
     */
    public static function replace($str, $replace)
    {

        $pattern = '';
        $default = ["'", ",", "(", ")"];
        if(empty($replace)) {
            return $str;
        }

        if(!is_array($replace)) {

            //$replace = [];
            $replace = json_decode($replace,true);

        }
//        else if (is_null($replace) || is_object($replace)) {
//
//            return $str;
//        }

        $replace = array_merge(array_diff($replace, ["default"]), $default);



        $replace = array_map(function ($item){
            return '\\'.$item;
        }, $replace);


        $pattern  = '/(['. implode($replace) . ']){1}/ui';

        return self::half2full($str, $pattern);
    }
}